@@ -17,6 +17,8 @@ use syn::{Lit, LitStr, Path};
 
 mod kind;
 
+use crate::util::path_to_string;
+
 use self::kind::{ErrorKind, ErrorUnknownField};
 
 /// An alias of `Result` specific to attribute parsing.
@@ -63,15 +65,6 @@ pub struct Error {
     span: Option<Span>,
 }
 
-/// Transform a syn::Path to a readable String
-fn path_to_string(path: &syn::Path) -> String {
-    path.segments
-        .iter()
-        .map(|s| s.ident.to_string())
-        .collect::<Vec<String>>()
-        .join("::")
-}
-
 /// Error creation functions
 impl Error {
     pub(in crate::error) fn new(kind: ErrorKind) -> Self {
