@@ -9,7 +9,7 @@ use std::sync::Arc;
 
 use syn::{Expr, Lit, Meta, NestedMeta};
 
-use crate::{Error, Result};
+use crate::{util::path_to_string, Error, Result};
 
 /// Create an instance from an item in an attribute declaration.
 ///
@@ -238,25 +238,6 @@ macro_rules! from_meta_float {
 from_meta_float!(f32);
 from_meta_float!(f64);
 
-/// Parsing support for identifiers. This attempts to preserve span information
-/// when available, but also supports parsing strings with the call site as the
-/// emitted span.
-impl FromMeta for syn::Ident {
-    fn from_string(value: &str) -> Result<Self> {
-        Ok(syn::Ident::new(value, ::proc_macro2::Span::call_site()))
-    }
-
-    fn from_value(value: &Lit) -> Result<Self> {
-        if let Lit::Str(ref ident) = *value {
-            ident
-                .parse()
-                .map_err(|_| Error::unknown_lit_str_value(ident))
-        } else {
-            Err(Error::unexpected_lit_type(value))
-        }
-    }
-}
-
 /// Parsing support for punctuated. This attempts to preserve span information
 /// when available, but also supports parsing strings with the call site as the
 /// emitted span.
@@ -272,19 +253,32 @@ impl<T: syn::parse::Parse, P: syn::parse::Parse> FromMeta for syn::punctuated::P
     }
 }
 
-/// Parsing support for an array, i.e. `example = "[1 + 2, 2 - 2, 3 * 4]"`.
-impl FromMeta for syn::ExprArray {
-    fn from_value(value: &Lit) -> Result<Self> {
-        if let Lit::Str(ref ident) = *value {
-            ident
-                .parse::<syn::ExprArray>()
-                .map_err(|_| Error::unknown_lit_str_value(ident))
-        } else {
-            Err(Error::unexpected_lit_type(value))
+macro_rules! from_syn_parse {
+    ($ty:path) => {
+        impl FromMeta for $ty {
+            fn from_string(value: &str) -> Result<Self> {
+                syn::parse_str(value).map_err(|_| Error::unknown_value(value))
+            }
+
+            fn from_value(value: &::syn::Lit) -> Result<Self> {
+                if let ::syn::Lit::Str(ref v) = *value {
+                    v.parse::<$ty>()
+                        .map_err(|_| Error::unknown_lit_str_value(v))
+                } else {
+                    Err(Error::unexpected_lit_type(value))
+                }
+            }
         }
-    }
+    };
 }
 
+from_syn_parse!(syn::Ident);
+from_syn_parse!(syn::Expr);
+from_syn_parse!(syn::ExprArray);
+from_syn_parse!(syn::ExprPath);
+from_syn_parse!(syn::Path);
+from_syn_parse!(syn::WhereClause);
+
 macro_rules! from_numeric_array {
     ($ty:ident) => {
         /// Parsing an unsigned integer array, i.e. `example = "[1, 2, 3, 4]"`.
@@ -314,24 +308,6 @@ from_numeric_array!(u32);
 from_numeric_array!(u64);
 from_numeric_array!(usize);
 
-/// Parsing support for paths. This attempts to preserve span information when available,
-/// but also supports parsing strings with the call site as the emitted span.
-impl FromMeta for syn::Path {
-    fn from_string(value: &str) -> Result<Self> {
-        syn::parse_str(value).map_err(|_| Error::unknown_value(value))
-    }
-
-    fn from_value(value: &Lit) -> Result<Self> {
-        if let Lit::Str(ref path_str) = *value {
-            path_str
-                .parse()
-                .map_err(|_| Error::unknown_lit_str_value(path_str))
-        } else {
-            Err(Error::unexpected_lit_type(value))
-        }
-    }
-}
-
 impl FromMeta for syn::Lit {
     fn from_value(value: &Lit) -> Result<Self> {
         Ok(value.clone())
@@ -367,17 +343,23 @@ impl FromMeta for syn::Meta {
     }
 }
 
-impl FromMeta for syn::WhereClause {
-    fn from_string(value: &str) -> Result<Self> {
-        syn::parse_str(value).map_err(|_| Error::unknown_value(value))
-    }
-}
-
 impl FromMeta for Vec<syn::WherePredicate> {
     fn from_string(value: &str) -> Result<Self> {
         syn::WhereClause::from_string(&format!("where {}", value))
             .map(|c| c.predicates.into_iter().collect())
     }
+
+    fn from_value(value: &Lit) -> Result<Self> {
+        if let syn::Lit::Str(s) = value {
+            syn::WhereClause::from_value(&syn::Lit::Str(syn::LitStr::new(
+                &format!("where {}", s.value()),
+                value.span(),
+            )))
+            .map(|c| c.predicates.into_iter().collect())
+        } else {
+            Err(Error::unexpected_lit_type(value))
+        }
+    }
 }
 
 impl FromMeta for ident_case::RenameRule {
@@ -432,14 +414,6 @@ impl<T: FromMeta> FromMeta for RefCell<T> {
     }
 }
 
-fn path_to_string(path: &syn::Path) -> String {
-    path.segments
-        .iter()
-        .map(|s| s.ident.to_string())
-        .collect::<Vec<String>>()
-        .join("::")
-}
-
 /// Trait to convert from a path into an owned key for a map.
 trait KeyFromPath: Sized {
     fn from_path(path: &syn::Path) -> Result<Self>;
@@ -452,7 +426,7 @@ impl KeyFromPath for String {
     }
 
     fn to_display(&self) -> Cow<'_, str> {
-        Cow::Borrowed(&self)
+        Cow::Borrowed(self)
     }
 }
 
@@ -605,6 +579,7 @@ mod tests {
     }
 
     #[test]
+    #[allow(clippy::bool_assert_comparison)]
     fn bool_succeeds() {
         // word format
         assert_eq!(fm::<bool>(quote!(ignore)), true);
@@ -790,6 +765,20 @@ mod tests {
         fm::<syn::ExprArray>(quote!(ignore = "[\"Hello World\", \"Test Array\"]"));
     }
 
+    #[test]
+    fn test_expr() {
+        fm::<syn::Expr>(quote!(ignore = "x + y"));
+        fm::<syn::Expr>(quote!(ignore = "an_object.method_call()"));
+        fm::<syn::Expr>(quote!(ignore = "{ a_statement(); in_a_block }"));
+    }
+
+    #[test]
+    fn test_expr_path() {
+        fm::<syn::ExprPath>(quote!(ignore = "std::mem::replace"));
+        fm::<syn::ExprPath>(quote!(ignore = "x"));
+        fm::<syn::ExprPath>(quote!(ignore = "example::<Test>"));
+    }
+
     #[test]
     fn test_number_array() {
         assert_eq!(
